@@ -1,8 +1,6 @@
-# -*- coding: utf-8 -*-
-#
 # This file is part of Linux Show Player
 #
-# Copyright 2012-2016 Francesco Ceruti <ceppofrancy@gmail.com>
+# Copyright 2016 Francesco Ceruti <ceppofrancy@gmail.com>
 #
 # Linux Show Player is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,18 +16,13 @@
 # along with Linux Show Player.  If not, see <http://www.gnu.org/licenses/>.
 
 
-# In the current state set/get_backend are quite useless
-__backend = None
+from .backend import Backend
 
 
-def set_backend(backend):
-    global __backend
-    __backend = backend
+def set_backend(backend: Backend):
+    global CurrentBackend
+    CurrentBackend = backend
 
 
-def get_backend():
-    """
-    :rtype: lisp.backends.base.backend.Backend
-    """
-    return __backend
-
+def get_backend() -> Backend:
+    return CurrentBackend
