@@ -1,9 +1,6 @@
-
-# -*- coding: utf-8 -*-
-#
 # This file is part of Linux Show Player
 #
-# Copyright 2012-2016 Francesco Ceruti <ceppofrancy@gmail.com>
+# Copyright 2022 Francesco Ceruti <ceppofrancy@gmail.com>
 #
 # Linux Show Player is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,7 +16,6 @@
 # along with Linux Show Player.  If not, see <http://www.gnu.org/licenses/>.
 
 import logging
-import traceback
 from functools import wraps, partial
 from threading import Thread, Lock, RLock
 
@@ -27,7 +23,7 @@ from threading import Thread, Lock, RLock
 def async_function(target):
     """Decorator. Make a function asynchronous.
 
-    The decorated function is executed in a differed thread.
+    The decorated function is executed in a different thread.
     """
 
     @wraps(target)
@@ -76,8 +72,9 @@ def locked_function(target=None, *, lock=None, blocking=True, timeout=-1):
 
     # If called with (keywords) arguments
     if target is None:
-        return partial(locked_function, lock=lock, blocking=blocking,
-                       timeout=timeout)
+        return partial(
+            locked_function, lock=lock, blocking=blocking, timeout=timeout
+        )
 
     if lock is None:
         target.__lock__ = RLock()
@@ -101,7 +98,7 @@ def locked_function(target=None, *, lock=None, blocking=True, timeout=-1):
 
 
 def locked_method(target=None, *, blocking=True, timeout=-1):
-    """Decorator. Make a *method* synchronized.
+    """Decorator. Make a *method* "synchronized".
 
     Only one thread at time can access the decorated method.
 
@@ -115,7 +112,7 @@ def locked_method(target=None, *, blocking=True, timeout=-1):
         return partial(locked_method, blocking=blocking, timeout=timeout)
 
     # generate a lock_name like "__method_name_lock__"
-    lock_name = '__' + target.__name__ + '_lock__'
+    lock_name = "__" + target.__name__ + "_lock__"
     target.__meta_lock__ = Lock()
 
     @wraps(target)
@@ -156,7 +153,7 @@ def suppress_exceptions(target=None, *, log=True):
         try:
             return target(*args, **kwargs)
         except Exception:
-            logging.warning('Exception suppressed:\n' + traceback.format_exc())
+            logging.warning("Exception suppressed.", exc_info=True)
 
     return wrapped
 
