@@ -1,9 +1,6 @@
-
-# -*- coding: utf-8 -*-
-#
 # This file is part of Linux Show Player
 #
-# Copyright 2012-2016 Francesco Ceruti <ceppofrancy@gmail.com>
+# Copyright 2016 Francesco Ceruti <ceppofrancy@gmail.com>
 #
 # Linux Show Player is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -32,7 +29,9 @@ from PyQt5.QtWidgets import QApplication
 from lisp.core.decorators import async_function
 from lisp.core.util import weak_call_proxy
 
-__all__ = ['Signal', 'Connection']
+__all__ = ["Signal", "Connection"]
+
+logger = logging.getLogger(__name__)
 
 
 def slot_id(slot_callable):
@@ -58,7 +57,7 @@ class Slot:
         elif callable(slot_callable):
             self._reference = weakref.ref(slot_callable, self._expired)
         else:
-            raise TypeError('slot must be callable')
+            raise TypeError("slot must be callable")
 
         self._callback = callback
         self._slot_id = slot_id(slot_callable)
@@ -72,14 +71,20 @@ class Slot:
                     self._reference()()
                 else:
                     self._reference()(*args, **kwargs)
-        except Exception:
-            logging.error(traceback.format_exc())
+        except Exception as e:
+            logger.warning(str(e), exc_info=True)
 
     def is_alive(self):
         return self._reference() is not None
 
-    def _expired(self, reference):
-        self._callback(self._slot_id)
+    def _expired(self, _):
+        if self._callback is not None:
+            self._callback(self._slot_id)
+
+    def __str__(self):
+        return (
+            f"{self.__class__.__qualname__}: {self._reference().__qualname__}"
+        )
 
 
 class AsyncSlot(Slot):
@@ -102,8 +107,9 @@ class QtSlot(Slot):
         self._invoker.customEvent = self._custom_event
 
     def call(self, *args, **kwargs):
-        QApplication.instance().sendEvent(self._invoker,
-                                          self._event(*args, **kwargs))
+        QApplication.instance().sendEvent(
+            self._invoker, self._event(*args, **kwargs)
+        )
 
     def _event(self, *args, **kwargs):
         return QSlotEvent(self._reference, *args, **kwargs)
@@ -116,8 +122,9 @@ class QtQueuedSlot(QtSlot):
     """Qt queued (safe) slot, execute the call inside the qt-event-loop."""
 
     def call(self, *args, **kwargs):
-        QApplication.instance().postEvent(self._invoker,
-                                          self._event(*args, **kwargs))
+        QApplication.instance().postEvent(
+            self._invoker, self._event(*args, **kwargs)
+        )
 
 
 class QSlotEvent(QEvent):
@@ -132,6 +139,7 @@ class QSlotEvent(QEvent):
 
 class Connection(Enum):
     """Available connection modes."""
+
     Direct = Slot
     Async = AsyncSlot
     QtDirect = QtSlot
@@ -176,14 +184,18 @@ class Signal:
         :raise ValueError: if mode not in Connection enum
         """
         if mode not in Connection:
-            raise ValueError('invalid mode value: {0}'.format(mode))
+            raise ValueError(f"invalid mode value: {mode}")
 
         with self.__lock:
-            # Create a new Slot object, use a weakref for the callback
-            # to avoid cyclic references.
-            callback = weak_call_proxy(weakref.WeakMethod(self.__remove_slot))
-            self.__slots[slot_id(slot_callable)] = mode.new_slot(slot_callable,
-                                                                 callback)
+            sid = slot_id(slot_callable)
+            # If already connected do nothing
+            if sid not in self.__slots:
+                # Create a new Slot object, use a weakref for the callback
+                # to avoid cyclic references.
+                self.__slots[sid] = mode.new_slot(
+                    slot_callable,
+                    weak_call_proxy(weakref.WeakMethod(self.__remove_slot)),
+                )
 
     def disconnect(self, slot=None):
         """Disconnect the given slot, or all if no slot is specified.
