@@ -1,8 +1,6 @@
-# -*- coding: utf-8 -*-
-#
 # This file is part of Linux Show Player
 #
-# Copyright 2012-2016 Francesco Ceruti <ceppofrancy@gmail.com>
+# Copyright 2016 Francesco Ceruti <ceppofrancy@gmail.com>
 #
 # Linux Show Player is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,16 +18,18 @@
 from os.path import dirname
 
 from lisp.core.loading import load_classes
-from lisp.ui.settings.settings_page import SettingsPage
 
 Protocols = []
-ProtocolsSettingsPages = []
+CueSettingsPages = []
+LayoutSettingsPages = []
 
 
 def load():
-    for _, protocol in load_classes(__package__, dirname(__file__),
-                                    pre=('', ''), suf=('', 'Settings',)):
-        if issubclass(protocol, SettingsPage):
-            ProtocolsSettingsPages.append(protocol)
-        else:
-            Protocols.append(protocol)
+    for _, protocol in load_classes(__package__, dirname(__file__)):
+        Protocols.append(protocol)
+
+        # Get settings pages
+        if protocol.CueSettings is not None:
+            CueSettingsPages.append(protocol.CueSettings)
+        if protocol.LayoutSettings is not None:
+            LayoutSettingsPages.append(protocol.LayoutSettings)
