@@ -1,8 +1,6 @@
-# -*- coding: utf-8 -*-
-#
 # This file is part of Linux Show Player
 #
-# Copyright 2012-2016 Francesco Ceruti <ceppofrancy@gmail.com>
+# Copyright 2016 Francesco Ceruti <ceppofrancy@gmail.com>
 #
 # Linux Show Player is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -22,12 +20,11 @@ from PyQt5.QtWidgets import QLabel
 
 
 class QClickLabel(QLabel):
-
     clicked = pyqtSignal(QEvent)
 
     def __init(self, parent):
         super().__init__(parent)
 
     def mouseReleaseEvent(self, e):
-        if(self.contentsRect().contains(e.pos())):
+        if self.contentsRect().contains(e.pos()):
             self.clicked.emit(e)
