@@ -1,8 +1,6 @@
-# -*- coding: utf-8 -*-
-#
 # This file is part of Linux Show Player
 #
-# Copyright 2012-2016 Francesco Ceruti <ceppofrancy@gmail.com>
+# Copyright 2016 Francesco Ceruti <ceppofrancy@gmail.com>
 #
 # Linux Show Player is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,8 +16,7 @@
 # along with Linux Show Player.  If not, see <http://www.gnu.org/licenses/>.
 
 from PyQt5.QtCore import Qt, pyqtSignal
-from PyQt5.QtWidgets import QSlider
-from PyQt5.QtWidgets import QStyle, QStyleOptionSlider
+from PyQt5.QtWidgets import QSlider, QStyle, QStyleOptionSlider
 
 
 class QClickSlider(QSlider):
@@ -53,5 +50,6 @@ class QClickSlider(QSlider):
         opt = QStyleOptionSlider()
         self.initStyleOption(opt)
 
-        return self.style().subControlRect(QStyle.CC_Slider, opt,
-                                           QStyle.SC_SliderHandle)
+        return self.style().subControlRect(
+            QStyle.CC_Slider, opt, QStyle.SC_SliderHandle
+        )
